"""
Hessian测试
"""

import numpy as np
from redundant_coordinate import RedundantCoordinate
from Gaussian09 import Gaussian09
from bond_connect import BondConnect


class EstimateHessian():
    def __init__(self):
        self.calculator = Gaussian09()
        self.ric = RedundantCoordinate()
        self.force_counter = 0
        self.svd_inv_thresh = 3.16e-4,
        self.BC = BondConnect()
        self.radius = self.BC.radius
        self.Bohr2Angs = 0.529177249

    def get_alpha_r(self, ele1, ele2):
        """
        Lind方案中的alpha值和r值计算
        """
        period_1 = ['H', 'He']
        period_2 = ['Li', 'Be', 'B', 'C', 'N', 'O', 'F', 'Ne']
        alpha = 1.0
        r = 1.35
        if ele1 in period_1 and ele2 in period_1:
            alpha = 1.0
            r = 1.35
        elif ele1 in period_1 or ele2 in period_1:
            alpha = 0.3949
            if ele1 in period_2 or ele2 in period_2:
                r = 2.1
            else:
                r = 2.53
        else:
            alpha = 0.28
            if ele1 in period_2 and ele2 in period_2:
                r = 2.87
            else:
                r = 3.4
        return (alpha, r)

    def guess_hess(self, coord, elements, connect):
        """
        估计Hessian值,在冗余内坐标下,使用Lindh估计方案
        Ref: https://www.sciencedirect.com/science/article/pii/000926149500646L
        """
        coord = coord.reshape(-1, 3)
        ric_index = self.ric.get_ric_index_list_prime(coord, connect)
        hessian_size = sum([len(item) for item in ric_index])
        estimat_hess = np.zeros((hessian_size, hessian_size), 'f')
        # k值分为键长 键角 二面角项
        k_list = [0.45, 0.15, 0.005]
        diag_hess = []
        for index_list in ric_index:
            # 分辨遍历键长 键角 二面角的索引表
            for index in index_list:
                # 使用k值初始化rho,k值分为键长 键角 二面角项
                rho = k_list[len(index)-2]
                for i in range(len(index)-1):
                    element_1 = elements[index[i]]
                    element_2 = elements[index[i+1]]
                    # Lind方案中的alpha值
                    alpha, r = self.get_alpha_r(element_1, element_2)
                    # 参考距离
                    r_ref = (self.radius[element_1] + self.radius[element_2]) / self.Bohr2Angs
                    # 真实距离
                    r_real = np.linalg.norm(coord[index[i]]-coord[index[i+1]])
                    rho *= np.exp(alpha * (r**2 - r_real**2))
                diag_hess.append(rho)
        return np.array(np.diag(diag_hess), np.float64)


if __name__ == "__main__":
    estHess = EstimateHessian()
    # estHess.ric_hess_trans_test()
    # estHess.guess_hess_test()
    # estHess.guess_test()
    estHess.mod_hess()
    path = r'D:\gaussian\estimate_Hessian\1_ts.fchk'
    estHess.read_fchk_hessian(path, 3)
